from django.contrib import admin
from .models import City

class CityAdmin(admin.ModelAdmin):
    list_display = ('name', 'user', 'is_actived', 'created', 'updated')
    list_per_page = 12
    list_filter = ['is_actived', 'created', 'updated']
    search_fields = ['name']


class CityAdminMixin(admin.ModelAdmin):
    """
    City auth admin
    """
    city = None

    def set_city(self, request):
        try:
            city = City.objects.get(user=request.user)
        except City.DoesNotExist:
            city = None
        self.city = city

    def get_list_display(self, request):
        self.set_city(request)
        if self.city:
            return self.list_display
        else:
            return ('city',) + self.list_display

    def get_list_filter(self, request):
        self.set_city(request)
        if self.city:
            return self.list_filter
        else:
            return ('city',) + self.list_filter

    def get_fields(self, request, obj=None):
        self.set_city(request)
        if self.city:
            return self.fields
        else:
            return ('city',) + self.fields

    def get_queryset(self, request):
        self.set_city(request)
        queryset = super(CityAdminMixin, self).get_queryset(request)
        if self.city:
            queryset = queryset.filter(city=self.city)
        return queryset

    def save_model(self, request, obj, form, change):
        self.set_city(request)
        if self.city:
            obj.city = self.city
        obj.save()

admin.site.register(City, CityAdmin)



import json
import urllib
from django.views.generic.base import View
from .models import City

class CityViewMixin(View):
    """
    City view mixin, set current city
    """
    def dispath(self, request, *args, **kwargs):
        try:
            city_name = request.session['city']
        except KeyError:
	    ip = get_ip(request)
            city_name = get_city_name(ip)
            request.session['city'] = city_name
        self.city = self.get_city(city_name)
        return super(CityViewMixin, self).dispatch(request, *args, **kwargs)

    def get_ip(self, request):
        """get client ip"""
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[0]
        else:
            ip = request.META.get('REMOTE_ADDR')
        return ip

    def get_city_name(self, ip):
        """get city """
        url = 'http://restapi.amap.com/v3/ip?ip=%s&key=%s'\
                % (ip,'bf3561891ae40b0696600106787ebe26')
        with urllib.urlopen(url) as f:
            data = f.read().decode('utf-8')
            json_data = json.loads(data)
            if json_data['status'] == 1:
                city = json_data['city'][0:-1]
            else:
                city = ''
            return city

    def get_city(self, name):
        """get city model"""
        try:
            city = City.objects.get(name=name)
        except City.DoesNotExist:
            city = City.objects.first()
        return city



from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.db import models

class City(models.Model):
    name = models.CharField(_('city'), max_length=200)
    user = models.OneToOneField(User, related_name='city',\
            verbose_name=_('user'))
    is_actived = models.BooleanField(_('is actived'), default=True)
    created = models.DateTimeField(_('created'), auto_now_add=True)
    updated = models.DateTimeField(_('updated'), auto_now=True)

    def __unicode__(self):
        return self.name

    class Meta(object):
        verbose_name = _('city')
        verbose_name_plural = _('city')
